<?php

/*

Simple PHP class to create 'pocketmod' style booklets from Google
(MyMaps) KML files.

http://www.aaronland.info/php/mmPDF/
Copyright (c) 2007 Aaron Straup Cope.

This is free software, you may use it and distribute it under the
same terms as Perl itself.

$Id: mmPDF.php,v 1.8 2007/06/05 04:30:02 asc Exp $

*/
# 

	class mmPDF extends pmPDF {

			function mmPDF($args=array()){

				$this->_places = array();
				$this->_page = 1;
				$this->_appid = $args['appid'];
				$this->_qr = $args['qr'];
				$this->_singlemap = $args['simple'];

				$this->_index = array();
				$this->pmPDF($args);
			}

			function add_map($file){

				$xml = simplexml_load_file($file);

				$count = 0;

				foreach ($xml->Document->Placemark as $pl){

					$name = trim($pl->name);
					$desc = str_replace("<br>", "\n", trim($pl->description));		
					$desc = preg_replace("/<a.*/", "", $desc);				       
	
					$key = md5($name . $desc);

					if (array_key_exists($key, $this->_places)){
						continue;
					}

					list($lon, $lat) = explode(",", $pl->Point->coordinates);

					# error_log("$lat $lon");

					$this->add_place($name, $desc, $lat, $lon);
					$this->_places[$key] ++;
					
					$count ++;

					if ($count == 30){
						break;
					}

					sleep(1);
				}


			}

			function Output($name='', $dest=''){

				$this->add_index($this->_index);
				parent::Output($name, $dest);
				return;
			}

			function add_index(&$idx){

				ksort($dix);

				foreach ($idx as $n => $pgs){

					$n .= " ";

					foreach ($pgs as $p){

						$nlen = $this->GetStringWidth($n);
						$plen = $this->GetStringWidth(" {$p}");				      

						$sect = $this->_sect_width;
						$limit = $sect - $plen;

						$str_len = ($nlen + $plen);
					
						while ($limit > ($str_len + $plen)){

							$n .= ".";
							$nlen = $this->GetStringWidth($n);
							$str_len = ($nlen + " " + $plen);
						}

						$res = $this->add_text("{$n} {$p}", $this->_page);
						$this->_page = $res['page'];
					}
				}

				return 1;
			}

			function add_place($name, $desc, $lat, $lon){

				if (! $this->_singlemap){

					while ($this->_page % 2){
						$this->_page ++;
					}

					$map1 = $this->get_img($lat, $lon, 4);

					$res = $this->add_image($map1, $this->_page);

					if ($this->_qr){
						$this->add_qr($name, $desc, $lat, $lon);
					}
					
					$this->_page = $res['page'] + 1;
					unlink($map1);
				}

				$map2 = $this->get_img($lat, $lon, 1);
				
				$res = $this->add_image($map2, $this->_page);
				$this->_page = $res['page'];

				$txt = "{$name}\n{$desc}";
				$this->add_text($txt, $this->_page, array('y' => 2.5, 'resume' => false));

				# 

				if (! array_key_exists($name, $this->_index)){
					$this->_index[$name] = array();
				}

				$this->_index[$name][] = $this->_page;
				$this->_page = $res['page'] + 1;

				# 

				unlink($map2);
			}

			function add_qr ($name, $desc, $lat, $lon){

				$qr_addr_path = $this->tempfile("qr_geo");				
				$qr_addr = new QR($this->_qr);
				
				$qr_addr->encode(array(
						       'd' => $desc,
						       'path' => $qr_addr_path,
						       'color' => array(0, 0, 51),
						       ));
				
				$this->add_image($qr_addr_path, $this->_page, array('y' => 2.5, 'h' => .75));
				
				if ($this->_qr['edit']){
					
					$qr_edit_path = $this->tempfile("qr_edit");
					$qr_edit = new QR($this->_qr);
					
					$qr_edit->encode(array(
							       'd' => "{$this->_qr['edit']}/{$name}",
							       'path' => $qr_edit_path,
							       'color' => array(84, 84, 84),
							       ));
					
					$this->add_image($qr_edit_path, $this->_page, array('y' => 2.5, 'x' => 1.5, 'h' => .75));
				}
				
				if ($this->_qr['geo']){
					
					$qr_geo_path = $this->tempfile("qr_geo");
					$qr_geo = new QR($this->_qr);
					
					$qr_geo->encode(array(
							      'd' => "$lat, $lon",
							      'path' => $qr_geo_path,
							      'color' => array(69, 139, 0),
							      ));
					
					
					$this->add_image($qr_geo_path, $this->_page, array('y' => 2.5, 'x' => .75, 'h' => .75));
				}
				
			}
			
			function tempfile ($suffix, $ext="png"){
				return tempnam("/tmp", time()) . "-{$suffix}.{$ext}";
			}

			function get_img($lat, $lon, $zoom){
				
				$req = "http://api.local.yahoo.com/MapsService/V1/mapImage?appid={$this->_appid}&latitude={$lat}&longitude={$lon}&zoom={$zoom}&image_width=500&image_type=png";
				
				$res = $this->simple_get($req);
				$xml = new SimpleXMLElement($res);
	
				if (! $xml){
					return;
				}
		
				$tmp = $this->tempfile($zoom);

				$fh  = fopen($tmp, "w");
				fwrite($fh, $this->simple_get($xml));
				fclose($fh);
				
				return $tmp;
			}


	}
?>